#version 430 core

layout(local_size_x = 512) in;
layout(binding = 0, rgba32f)  readonly uniform image2D inImage;
layout(binding = 1, rgba32f) writeonly uniform image2D outImage;

shared vec3 shData[gl_WorkGroupSize.x * 2];

void main() {
	
	uint	id = gl_LocalInvocationID.x;
	ivec2	texCoord = ivec2(id * 2, gl_WorkGroupID.x);
	uint	i0 = id * 2;
	uint	i1 = i0 + 1;

	//	Co-operative read in
	shData[i0] = imageLoad(inImage, texCoord).rgb;
	shData[i1] = imageLoad(inImage, texCoord + ivec2(1, 0)).rgb;

	barrier();
	memoryBarrierShared();

	const uint loopCount = uint(log2(gl_WorkGroupSize.x)) + 1;

	for (int step = 0; step < loopCount; step++) {

		// offset = 0, 1, 3, 7, 15 ....
		uint offset = (1 << step) - 1;
		uint rIndex = ((id >> step) << (step + 1)) + offset;
		uint wIndex = rIndex + (id & offset) + 1;
		shData[wIndex].rgb += shData[rIndex].rgb;

		barrier();
		memoryBarrierShared();

	}

	//	Co-operative write back
	imageStore(outImage, texCoord.yx, vec4(shData[i0].rgb, 1.0));
	imageStore(outImage, texCoord.yx + ivec2(0,1), vec4(shData[i1].rgb, 1.0));

}
